# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Writing data to IO text buffer using queue and working in separate
thread."""


from collections import deque
from threading import Thread, Lock, Event
from time import sleep
from warnings import warn


class QueuedWriter:
    """Tool for record data to text buffer using queue and thread."""
    def __init__(self, buff=None, write_on='time', write_every=1):
        self._buff = buff
        self._queue = deque([])
        self._thread = None
        self._thread_args = (self._save_queue, write_on, write_every)
        self._convert_func = lambda x: x

    def set_buff(self, buff):
        """Set IO buffer."""
        self._buff = buff

    def set_convert_func(self, func):
        """Set function for convertion data sample to string."""
        self._convert_func = func

    def start_thread(self):
        """Start recording thread."""
        self._thread = PyRecordThread(*self._thread_args)
        self._thread.start()

    def start_record(self):
        """Deprecated."""
        warn("start_record() is deprecated. Use start_thread().")
        self.start_thread()

    def add_data(self, sample):
        """Add data sample."""
        self._queue.append(sample)
        self._thread.write_data()

    def _save_queue(self):
        """Move data from queue to buffer."""
        while self._queue:
            sample = self._queue.popleft()
            line = self._convert_func(sample)
            self._buff.write(line)

    def stop_thread(self):
        """Stop recording record."""
        self._thread.write_data()
        self._thread.stop()

    def stop_record(self):
        """Deprecated."""
        warn("stop_record() is deprecated. Use stop_thread().")
        self.stop_thread()


class PyRecordThread(Thread):
    """Thread in which samples recorded to IO buffer."""
    def __init__(self, save_func, write_on='time', write_every=1):
        super().__init__()
        self._lock = Lock()
        self._must_write = False
        self._can_quit = Event()
        self._have_data = Event()
        self.save_func = save_func
        if write_on == 'time':
            self._loop_func = self._sleep_loop
            self._loop_func_args = (write_every, )
        elif write_on == 'data':
            self._loop_func = self._event_loop
            self._loop_func_args = ()
        else:
            raise RuntimeError("write_on must be 'time' or 'data'.")

    def run(self):
        """Record data from queue to buffer."""
        self._can_quit.clear()
        with self._lock:
            self._must_write = True
        self._loop_func(*self._loop_func_args)
        self._can_quit.set()

    def _sleep_loop(self, period):
        while self._must_write:
            sleep(period)
            self.save_func()

    def _event_loop(self):
        while self._must_write:
            self._have_data.clear()
            self._have_data.wait()
            self.save_func()

    def write_data(self):
        """Ask thread to write data."""
        self._have_data.set()

    def stop(self):
        """Stop thread."""
        with self._lock:
            self._must_write = False
        self._can_quit.wait()
        self.join()


def _example():
    from random import randint

    def convert_func(sample):
        """Convert function."""
        return '{}: {}\n'.format(*sample)

    writer = QueuedWriter(write_on='time')
    writer.set_convert_func(convert_func)
    buf = open('output.txt', 'w')
    writer.set_buff(buf)
    writer.start_thread()

    i = 0
    while True:
        try:
            sample = (i, randint(0, 100))
            print(sample)
            writer.add_data(sample)
            sleep(0.02)
            i += 1
        except KeyboardInterrupt:
            print("Good bye...")
            break

    writer.stop_thread()
    buf.close()


if __name__ == "__main__":
    _example()



import sys
import os

sys.path.insert(0, os.path.abspath("."))
from appteka.pyqt import testing
from appteka.pyqtgraph.waveform import Waveform


class TestWaveform(testing.TestDialog):
    """Test suite for Waveform widget."""
    def __init__(self):
        super().__init__()
        self.resize(600, 600)

    def test_xlabel(self):
        w = Waveform(self, "Time [sec]")
        self.set_widget(w)
        self.add_assertion("x label is 'Time [sec]'")

    def test_time_axis_false(self):
        w = Waveform(self, time_axis=False)
        self.set_widget(w)
        w.update_data([0, 1, 2, 3], [1, 2, 1, 2])
        self.add_assertion("x values are usual numbers from 0 to 3")

    def test_time_axis_true(self):
        w = Waveform(self, time_axis=True)
        self.set_widget(w)
        w.update_data([0, 1, 2, 3], [1, 2, 1, 2])
        self.add_assertion("x values are time values")

    def test_scaling(self):
        # Scenario: scaling with keys CONTROL and SHIFT
        w = Waveform(self)
        self.set_widget(w)
        w.update_data([0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
                      [1, 2, 0, 3, 1, 2, 3, 4, 1, 3])

        self.add_assertion("both axis scaling with mouse wheel")
        self.add_assertion("x-scaling with CONTROL pressed")
        self.add_assertion("y-scaling with SHIFT pressed")


testing.run(TestWaveform)



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Waveform widgets."""

import time
from warnings import warn

from PyQt5.QtCore import Qt

import pyqtgraph as pg


class TimeStampAxisItem(pg.AxisItem):
    """Axis with times or dates as ticks."""
    def __init__(self, what_show='time', *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.enableAutoSIPrefix(enable=False)
        if what_show not in ['time', 'date']:
            raise ValueError("show must be 'time' or 'date'")
        self.what_show = what_show

    def tickStrings(self, values, scale, spacing):
        if self.what_show == 'time':
            return [time.strftime("%H:%M:%S", time.gmtime(secs))
                    for secs in values]
        if self.what_show == 'date':
            return [time.strftime("%y-%m-%d", time.gmtime(secs))
                    for secs in values]
        raise RuntimeError("scale must be time or date")


def get_time_stamp_axis_item(top=True):
    """Return custom axis item with time stamps."""
    res = {
        'bottom': TimeStampAxisItem(
            what_show='time',
            orientation='bottom'
        )
    }
    if top:
        res['top'] = TimeStampAxisItem(
            what_show='date',
            orientation='top'
        )
    return res


class Waveform(pg.PlotWidget):
    """Customized PyQtGraph.PlotWidget."""
    def __init__(self, parent=None, xlabel=None, time_axis=True):
        if time_axis:
            axisItems = get_time_stamp_axis_item()
        else:
            axisItems = None
        super().__init__(parent, axisItems=axisItems)

        self.showAxis('top')
        self.showAxis('right')

        self.state = {
            'online': False,
            'plot_color': (255, 255, 255),
        }
        self.showGrid(x=True, y=True)

        if xlabel is not None:
            self.setLabel('bottom', xlabel)

        self.setDownsampling(mode='peak')
        self.setClipToView(True)
        self.enableAutoRange(True)
        self.curve = self.plot()

    def reset(self):
        """Clear plot."""
        self.clear()
        self.curve = self.plot()
        self.set_plot_color(self.state['plot_color'])
        self.setMouseEnabled(x=True, y=True)
        self.enableAutoRange(True)

    def set_online(self, value):
        """Set online or offline mode for waveform."""
        self.state['online'] = value
        if value:
            self.setClipToView(True)
            self.setMouseEnabled(x=False)
        else:
            self.setClipToView(False)
            self.setMouseEnabled(x=True)

    def set_plot_color(self, color):
        """Change plot color."""
        self.curve.setPen(color)
        self.state['plot_color'] = color

    def update_data(self, t, x):
        """Update plot."""
        if self.state['online'] and not self.isVisible():
            return
        if len(t) == 0:
            return
        self.curve.setData(t, x)
        self.setLimits(xMin=t[0], xMax=t[-1])
        self.setRange(
            xRange=(t[0], t[-1]),
            yRange=(min(x), max(x)),
            disableAutoRange=False
        )

    def keyPressEvent(self, ev):
        if ev.key() == Qt.Key_Shift:
            self.setMouseEnabled(x=False, y=True)

        if ev.key() == Qt.Key_Control:
            self.setMouseEnabled(x=True, y=False)

        super().keyPressEvent(ev)

    def keyReleaseEvent(self, ev):
        self.setMouseEnabled(x=True, y=True)
        super().keyReleaseEvent(ev)

    def leaveEvent(self, ev):
        self.setMouseEnabled(x=True, y=True)
        super().leaveEvent(ev)


class MultiWaveform(pg.GraphicsLayoutWidget):
    """Customized PyQtGraph.GraphicsLayoutWidget."""
    def __init__(self, parent=None):
        """Initialization."""
        super().__init__(parent)
        self.state = {
            'online': False,
            'plot_color': (255, 255, 255),
        }
        self.plots = {}
        self.curves = {}
        self._main = None
        self.__main_plot_limits = None

    def set_main_plot(self, key):
        """Set the plot used for synchronization (main plot)."""
        self._main = key

    def get_main_plot(self):
        """Set the plot used for synchronization (main plot)."""
        return self._main

    main_plot = property(
        get_main_plot,
        set_main_plot,
        doc="Plot used for synchronization"
    )

    def _init_plot(self, key):
        # TODO: move this code to add_plot
        """Bring plot to initial state."""
        if key not in self.plots.keys():
            return
        plot = self.plots[key]
        plot.setDownsampling(mode='peak')
        plot.showGrid(x=True, y=True)
        plot.showAxis('top')
        plot.showAxis('right')
        for axis in ['left', 'right']:
            plot.getAxis(axis).setStyle(
                tickTextWidth=40,
                autoExpandTextSpace=False
            )
        plot.enableAutoRange(True)

    def add_plot(self, key, title=None, main=False, time_axes=False):
        """Add plot."""
        if time_axes:
            axes = get_time_stamp_axis_item()
        else:
            axes = None

        self.plots[key] = self.addPlot(len(self.plots), 0, axisItems=axes)
        self._init_plot(key)

        if title is not None:
            self.plots[key].setTitle(title, justify='left')

        self.curves[key] = self.plots[key].plot()
        self.curves[key].setPen(self.state['plot_color'])

        # TODO: deprecate main argument
        if main:
            self._main = key

    def remove_plots(self):
        """Remove all plots."""
        for key in self.plots:
            self.removeItem(self.plots[key])
        self.plots = {}
        self.curves = {}
        self._main = None

    def update_data(self, key, t, x):
        """Update data on the plot."""
        if len(t) == 0:
            return
        if self.state['online'] and not self.isVisible():
            return

        xlims = (t[0], t[-1])

        if key == self._main:
            self.__main_plot_limits = xlims
        elif self._main is not None:
            xlims = self.__main_plot_limits

        self.curves[key].setData(t, x)

        self.plots[key].setLimits(xMin=xlims[0], xMax=xlims[1])
        self.plots[key].setRange(
            xRange=(xlims[0], xlims[1]),
            yRange=(min(x), max(x)),
            disableAutoRange=False
        )

    def set_online(self, value):
        """Turn on or turn off the online mode."""
        self.state['online'] = value
        if value:
            for plot in self.plots.values():
                plot.setClipToView(True)
                plot.setMouseEnabled(x=False)
        else:
            for plot in self.plots.values():
                plot.setClipToView(False)
                plot.setMouseEnabled(x=True)

    def set_plot_color(self, color):
        """Set color for all plots."""
        message = "MultiWaveform.set_plot_color() is deprecated. "
        message += "Use MultiWaveform.set_plots_color()."
        warn(message)
        self.set_plots_color(color)

    def set_plots_color(self, color):
        """Set color for all plots."""
        for curve in self.curves.values():
            curve.setPen(color)
        self.state['plot_color'] = color

    def set_link_to_main(self, value=True):
        """Link plots to main or unlink."""
        for key in self.plots:
            if key == self._main:
                continue
            self.plots[key].setXLink(self.plots[self._main] if value else None)

    def keyPressEvent(self, ev):
        if ev.key() == Qt.Key_Shift:
            for plot in self.plots.values():
                plot.setMouseEnabled(x=False, y=True)

        if ev.key() == Qt.Key_Control:
            for plot in self.plots.values():
                plot.setMouseEnabled(x=True, y=False)

        super().keyPressEvent(ev)

    def keyReleaseEvent(self, ev):
        for plot in self.plots.values():
            plot.setMouseEnabled(x=True, y=True)
        super().keyReleaseEvent(ev)

    def leaveEvent(self, ev):
        for plot in self.plots.values():
            plot.setMouseEnabled(x=True, y=True)
        super().leaveEvent(ev)



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


""" Helper functions for building Qt GUI. """

from PyQt5 import QtWidgets, QtGui


def add_action(window, name, slot, pic=None, shortcut=None, menu=None):
    """Add action conected with clot to the main window."""
    action = None
    if pic:
        action = QtWidgets.QAction(QtGui.QIcon(pic), name, window)
    else:
        action = QtWidgets.QAction(name, window)
    action.triggered.connect(slot)
    if shortcut:
        action.setShortcut(shortcut)
    if menu is not None:
        menu.addAction(action)
    return action


def add_sublayout(parent_layout, direction="h"):
    """Add sublayout."""
    if direction.lower() == "h":
        layout = QtWidgets.QHBoxLayout()
    elif direction.lower() == "v":
        layout = QtWidgets.QVBoxLayout()
    else:
        return None
    parent_layout.addLayout(layout)
    return layout


def add_button(text, slot, layout):
    """Add button connected with slot to layout."""
    button = QtWidgets.QPushButton(text)
    button.clicked.connect(slot)
    layout.addWidget(button)
    return button


def add_edit(layout):
    """Add line edit to layout."""
    edit = QtWidgets.QLineEdit()
    layout.addWidget(edit)
    return edit


def add_label(text, layout):
    """Add text label to layout."""
    label = QtWidgets.QLabel(text)
    layout.addWidget(label)
    return label


def add_widget(widget, layout):
    """Add widget to layout."""
    layout.addWidget(widget)
    return widget


def show_about(title="About program", name="", version="",
               descr="", parent=None):
    """Show about window."""
    mbox = QtWidgets.QMessageBox(parent)
    mbox.setWindowTitle(title)
    text = "<p><b>{} {}</b></p>".format(name, version)
    text += "<p> {} </p>".format(descr)
    mbox.setText(text)
    mbox.exec()



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Helper for TDD of sqlite3 schema."""

import sqlite3


class TestSchemaHelper:
    """This class holds sqlite3 connection and cursor and allows to test
    select queries.

    Use it within test_ functions of uinttest.TestCase.

    Parameters
    ----------
    test_case: unittest.TestCase
        Instance of test case.

    Examples
    --------
    >>> class TestMySchema(unittest.TestCase):
    >>>     def __init__(self):
    >>>         self.h = TestSchemaHelper()
    >>>         ...
    >>>
    >>>     def test_some_query(self):
    >>>         q = "SELECT ..."
    >>>         r = [(...), (...), ...]
    >>>         self.h.test_select(self, q, r)

    """
    def __init__(self, test_case):
        self._conn = sqlite3.connect(":memory:")
        self._cur = self._conn.cursor()
        self._test_case = test_case

    def test_select(self, query, ref):
        """Execute select query and compare result with reference one."""
        res = list(self._cur.execute(query))
        self._test_case.assertEqual(res, ref)

    def dot_read(self, path):
        """Execute all queries from path. See .read command of sqlite."""
        with open(path) as buf:
            script = buf.read()

        queries = script.split(";")

        for query in queries:
            self._cur.execute(query)

    def close(self):
        """Close connection with database."""
        self._conn.close()



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" This module implements the SimpleSignal class which makes the
event handling more convenient. """


class SimpleSignal:
    """Provides simple signals and slots mechanism."""
    def __init__(self):
        """Constructor.

        Create empty list of connected function.
        """
        self.slots = []

    def emit(self, *args):
        """Emit signal."""
        for s in self.slots:
            if s:
                s(args)

    def connect(self, slot):
        """Connect signal with slot.

        Parameters
        ----------
        slot
            Function to be connected with signal.
        """
        for i in range(len(self.slots)):
            if self.slots[i] == slot:
                return
            if self.slots[i] is None:
                self.slots[i] = slot
                return
        self.slots.append(slot)

    def disconnect(self, slot):
        """Disconnect slot from signal.

        Parameters
        ----------
        slot
            Name of connected function.
        """
        for i in range(len(self.slots)):
            if self.slots[i] == slot:
                self.slots[i] = None



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""State machine for enabling/disabling controls."""


class ControlsStateMachine:
    """ State machine for enable/disable/hide/show controls. """
    def __init__(self):
        """ Initialization. """
        self.__states = {}
        self.__controls = []

    def add_state(self, state_name):
        """ Add new state (or reset existing one). """
        self.__states[state_name] = {
            'invisible': [],
            'disabled': []
        }

    def setup_state(self, state_name, invisible=[], disabled=[]):
        """ Setup sate. """
        if state_name not in self.__states:
            self.add_state(state_name)
        self._setup_controls_state(invisible, state_name, 'invisible')
        self._setup_controls_state(disabled, state_name, 'disabled')

    def set_state(self, state_name):
        """ Set currect state. """
        for c in self.__controls:
            c.setVisible(c not in self.__states[state_name]['invisible'])
            c.setEnabled(c not in self.__states[state_name]['disabled'])

    def _setup_controls_state(self, controls, state_name, key):
        """ Helper function. """
        self.__states[state_name][key] = []
        for c in controls:
            if c not in self.__controls:
                self.__controls.append(c)
            self.__states[state_name][key].append(c)



import sys
import os

sys.path.insert(0, os.path.abspath("."))
from appteka.pyqt import testing
from appteka.pyqtgraph.waveform import MultiWaveform


class TestMultiWaveform(testing.TestDialog):
    """Test case for MultiWaveform widget."""
    def __init__(self):
        super().__init__()
        self.resize(600, 600)

    def test_top_axis(self):
        # Given multiwaveform
        w = MultiWaveform(self)
        self.set_widget(w)

        # When add plot
        w.add_plot('a', title='plot A')

        # Then there is top axis
        self.add_assertion("There is top axis")

    def test_scaling(self):
        # Scenario: scaling with keys CONTROL and SHIFT
        w = MultiWaveform(self)
        self.set_widget(w)

        w.add_plot('a', title='plot A')
        w.add_plot('b', title='plot B')

        w.update_data('a',
                      [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
                      [1, 2, 0, 3, 1, 2, 3, 4, 1, 3])

        w.update_data('b',
                      [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
                      [2, 1, 1, 1, 4, 6, 2, 3, 4, 3])

        self.add_assertion("both axis scaling with mouse wheel")
        self.add_assertion("x-scaling with CONTROL pressed")
        self.add_assertion("y-scaling with SHIFT pressed")


testing.run(TestMultiWaveform)



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Helpers for distribution of software."""


import sys
import platform
import locale
import gettext
from pkg_resources import resource_filename as resource


def build_folder_name(name, version, appendix=None):
    """Return name of build folder."""
    bit_version = platform.architecture()[0]
    os_name = sys.platform
    if 'win' in os_name:
        os_name = 'win'
    res = "build/{}-{}".format(name, version)
    if appendix is not None:
        res += "-{}".format(appendix)
    res += "-{}-{}".format(os_name, bit_version)
    return res


def init_translation(package_name, resource_name, module_name):
    """ Returns function for specifying of places to be translated. """
    lang = locale.getdefaultlocale()[0]
    gettext.install(module_name)

    def _echo(text):
        return text

    try:
        trans = gettext.translation(
            module_name,
            resource(package_name, resource_name),
            languages=[lang],
        )
        return trans.gettext
    except (ImportError, FileNotFoundError):
        return _echo



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = '0.8.5'



import os
import sys

from PyQt5 import QtCore

sys.path.insert(0, os.path.abspath("."))
from appteka.pyqt import testing
from appteka.pyqtgraph import phasor


class TestPhasor(testing.TestDialog):
    def test_just_visible(self):
        self.set_widget(phasor.PhasorDiagram())
        self.add_assertion("Widget seems to be OK")

    def test_radius_labels(self):
        d = phasor.PhasorDiagram()
        self.set_widget(d)
        self.add_assertion("Widget seems to be OK")

    def test_range_is_two(self):
        d = phasor.PhasorDiagram()
        self.set_widget(d)
        d.set_range(2)
        self.add_assertion("Grid is OK")
        self.add_assertion("The max circle in grid has radius of 2")

    def test_change_range(self):
        d = phasor.PhasorDiagram()
        self.set_widget(d)
        d.set_range(2)
        d.set_range(4)
        self.add_assertion("Grid is OK")
        self.add_assertion("The max circle in grid has radius of 4")

    def test_add_phasor(self):
        d = phasor.PhasorDiagram()
        self.set_widget(d)
        d.set_range(100)
        d.add_phasor('ph-1', 80, 1)
        self.add_assertion("There is a phasor in first quadrant")
        self.add_assertion("The color of phasor is white")

    def test_update_phasor(self):
        d = phasor.PhasorDiagram()
        self.set_widget(d)
        d.set_range(100)
        d.add_phasor('ph-1', 80, 1)
        d.update_phasor('ph-1', 80, 2)
        self.add_assertion("There is a phasor in second quadrant")
        self.add_assertion("Phasor in single")

    def test_phasor_color(self):
        d = phasor.PhasorDiagram()
        self.set_widget(d)
        d.set_range(100)
        d.add_phasor('ph-1', 80, 1, (255, 0, 0))
        self.add_assertion("There is a red phasor in first quadrant")

    def test_three_phasors(self):
        d = phasor.PhasorDiagram()
        self.set_widget(d)
        d.set_range(100)
        d.add_phasor('ph-1', 80, 0, (255, 0, 0))
        d.add_phasor('ph-2', 80, 2 * 3.1415 / 3, (0, 255, 0))
        d.add_phasor('ph-3', 80, -2 * 3.1415 / 3, (0, 0, 255))

        self.add_assertion("There are 3 phasors: red, green and blue")
        self.add_assertion("About 120 degrees between every two phasors")

    def test_three_phasors_rotated(self):
        d = phasor.PhasorDiagram()
        self.set_widget(d)
        d.set_range(100)
        d.add_phasor('ph-1', 80, 0, (255, 0, 0))
        d.add_phasor('ph-2', 80, 2 * 3.1415 / 3, (0, 255, 0))
        d.add_phasor('ph-3', 80, -2 * 3.1415 / 3, (0, 0, 255))
        d.update_phasor('ph-1', 80, 1)
        d.update_phasor('ph-2', 80, 1 + 2 * 3.1415 / 3)
        d.update_phasor('ph-3', 80, 1 - 2 * 3.1415 / 3)

        self.add_assertion("There are 3 phasors: red, green and blue")
        self.add_assertion("About 120 degrees between every two phasors")
        self.add_assertion("Red phasor has angle about 1 radian")

    def test_range_to_phasor(self):
        d = phasor.PhasorDiagram()
        self.set_widget(d)
        d.add_phasor('ph-1', color=(255, 255, 0))
        d.update_phasor('ph-1', 1, 1)
        d.update_phasor('ph-1', 100, 1)
        d.set_range(100)
        self.add_assertion("Grid corresponds to phasor")

    def test_legend(self):
        d = phasor.PhasorDiagram()
        self.set_widget(d)
        d.add_phasor('ph-1', 80, 0, (255, 0, 0))
        d.add_phasor('ph-2', 80, 2 * 3.1415 / 3, (0, 255, 0))
        d.add_phasor('ph-3', 80, -2 * 3.1415 / 3, (0, 0, 255))
        d.show_legend()
        d.set_range(80)
        self.add_assertion("Legend is correct")

    def test_three_phasors_animation(self):
        d = phasor.PhasorDiagram()
        self.set_widget(d)
        d.add_phasor('ph-1', color=(255, 0, 0))
        d.add_phasor('ph-2', color=(0, 255, 0))
        d.add_phasor('ph-3', color=(0, 0, 255))
        d.show_legend()

        self.timer = QtCore.QTimer(self)
        self.timer.setInterval(10)

        self.step = 0

        def rotate():
            a = 2 * 3.1415 / 3
            sh = self.step / 200
            ash = self.step / 10
            d.update_phasor('ph-1', ash + 10, sh)
            d.update_phasor('ph-2', 10, sh + a)
            d.update_phasor('ph-3', 10, sh - a)
            d.set_range(ash + 10)

            self.step += 1
            if self.step > 100:
                self.timer.stop()
                self.enable_buttons()

        self.timer.timeout.connect(rotate)
        self.timer.start()
        self.disable_buttons()

        self.add_assertion("Phasors smoothly rotating")
        self.add_assertion("Amplitude of red phasor grows")

    def test_width_of_phasors(self):
        # Given phasor diagram
        d = phasor.PhasorDiagram()
        self.set_widget(d)

        # When add two phasors
        # And widths of phasors are set to be significantly differ
        d.add_phasor('ph-1', color=(255, 255, 255), width=1)
        d.add_phasor('ph-2', color=(255, 255, 255), width=4)
        d.update_phasor('ph-1', 100, 1)
        d.update_phasor('ph-2', 100, 2)
        d.set_range(100)

        # Then widths of phasors are differ
        self.add_assertion("widths of phasors are differ")


testing.run(TestPhasor)



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""This module implements text edit with line numbers and highlighting
of current line."""

from PyQt5 import QtWidgets, QtGui
from PyQt5.QtCore import QSize, QRect, Qt


class CodeTextEdit(QtWidgets.QPlainTextEdit):
    """Text field for editing the source code."""
    def __init__(self, parent=None):
        super().__init__(parent)
        self.lineNumberArea = LineNumberArea(self)

        self.blockCountChanged.connect(self.updateLineNumberAreaWidth)
        self.updateRequest.connect(self.updateLineNumberArea)
        self.cursorPositionChanged.connect(self.highlightCurrentLine)

        self.updateLineNumberAreaWidth(0)
        self.highlightCurrentLine()

        f = QtGui.QFont("monospace")
        f.setStyleHint(QtGui.QFont.Monospace)
        self.setFont(f)

    def set_text(self, text):
        """Set text."""
        self.document().setPlainText(text)

    def lineNumberAreaPaintEvent(self, event):
        painter = QtGui.QPainter(self.lineNumberArea)
        painter.fillRect(event.rect(), Qt.lightGray)

        block = self.firstVisibleBlock()
        blockNumber = block.blockNumber()
        top = self.blockBoundingGeometry(block).translated(
            self.contentOffset()).top()
        bottom = top + self.blockBoundingRect(block).height()

        while block.isValid() and top <= event.rect().bottom():
            if block.isVisible() and bottom >= event.rect().top():
                number = "{}".format(blockNumber + 1)
                painter.setPen(Qt.black)
                painter.drawText(
                    0, top, self.lineNumberArea.width(),
                    self.fontMetrics().height(), Qt.AlignRight,
                    number
                )

            block = block.next()
            top = bottom
            bottom = top + self.blockBoundingRect(block).height()
            blockNumber = blockNumber + 1

    def lineNumberAreaWidth(self):
        digits = 1
        mx = max(1, self.blockCount())
        while mx >= 10:
            mx = mx / 10
            digits = digits + 1

        # ch = QLatin1Char('9')
        ch = '9'
        space = 3 + self.fontMetrics().horizontalAdvance(ch) * digits
        return space

    def resizeEvent(self, e):
        super().resizeEvent(e)
        cr = self.contentsRect()
        r = QRect(
            cr.left(),
            cr.top(),
            self.lineNumberAreaWidth(),
            cr.height()
        )
        self.lineNumberArea.setGeometry(r)

    def updateLineNumberAreaWidth(self, newBlockCount):
        self.setViewportMargins(self.lineNumberAreaWidth(), 0, 0, 0)

    def highlightCurrentLine(self):
        extraSelections = []
        if not self.isReadOnly():
            selection = QtWidgets.QTextEdit.ExtraSelection()
            lineColor = QtGui.QColor(Qt.yellow).lighter(160)
            selection.format.setBackground(lineColor)
            selection.format.setProperty(
                QtGui.QTextFormat.FullWidthSelection, True)
            selection.cursor = self.textCursor()
            selection.cursor.clearSelection()
            extraSelections.append(selection)

        self.setExtraSelections(extraSelections)

    def updateLineNumberArea(self, rect, dy):
        if dy:
            self.lineNumberArea.scroll(0, dy)
        else:
            self.lineNumberArea.update(
                0, rect.y(),
                self.lineNumberArea.width(),
                rect.height()
            )

        if rect.contains(self.viewport().rect()):
            self.updateLineNumberAreaWidth(0)


class LineNumberArea(QtWidgets.QWidget):
    def __init__(self, editor):
        super().__init__(editor)
        self.code_editor = editor

    def sizeHint(self):
        return QSize(self.code_editor.lineNumberAreaWidth(), 0)

    def paintEvent(self, event):
        self.code_editor.lineNumberAreaPaintEvent(event)



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Implementation of the phasor diagram."""

import math
import pyqtgraph as pg

DEFAULT_CIRCLES_NUM = 6


class PhasorDiagram(pg.PlotWidget):
    """Widget for plotting phasor diagram."""
    def __init__(self, parent=None, size=500):
        super().__init__(parent)
        self.setAspectLocked(True)
        self.addLine(x=0, pen=0.2)
        self.addLine(y=0, pen=0.2)
        self.showAxis('bottom', False)
        self.showAxis('left', False)

        # fix size
        self.setFixedSize(size, size)

        self.__build_grid()
        self.__build_labels()

        self.set_range(1)

        self.phasors = {}

        self.setMouseEnabled(x=False, y=False)
        self.disableAutoRange()
        self.plotItem.setMenuEnabled(False)
        self.hideButtons()

        self.legend = None

    def set_range(self, value):
        """Set range of diagram."""
        self.__update_grid(value)
        self.__update_labels(value)

    def __build_grid(self):
        self.circles = []
        for i in range(DEFAULT_CIRCLES_NUM):
            circle = pg.QtGui.QGraphicsEllipseItem()
            circle.setPen(pg.mkPen(0.2))
            self.circles.append(circle)
            self.addItem(circle)

    def __build_labels(self):
        self.labels = []
        for i in range(2):
            label = pg.TextItem()
            self.labels.append(label)
            self.addItem(label)

    def __update_grid(self, value):
        for i in range(DEFAULT_CIRCLES_NUM):
            r = (i + 1) * value / DEFAULT_CIRCLES_NUM
            self.circles[i].setRect(-r, -r, r*2, r*2)

        self.setRange(
            xRange=(-r, r),
            yRange=(-r, r),
            disableAutoRange=True
        )

    def __update_labels(self, value):
        self.labels[0].setText("{}".format(value / 2))
        self.labels[0].setPos(0, value / 2)
        self.labels[1].setText("{}".format(value))
        self.labels[1].setPos(0, value)

    def add_phasor(self, name, am=0, ph=0, color=(255, 255, 255), width=1):
        """Add phasor to the diagram."""
        phasor = {
            'end': (
                am * math.cos(ph),
                am * math.sin(ph)
            ),
            'line': self.plot(),
        }
        phasor['point'] = self.plot(pen=None, symbolBrush=color,
                                    symbolSize=width+5, symbolPen=None,
                                    name=name)
        phasor['line'].setPen(pg.mkPen(color, width=width))
        self.phasors[name] = phasor
        self.__update()

    def remove_phasors(self):
        """Remove phasors and legend."""
        for key in self.phasors:
            item = self.phasors[key]['point']
            self.removeItem(item)
            item = self.phasors[key]['line']
            self.removeItem(item)
        self.phasors = {}

        # remove legend
        if self.legend is not None:
            self.legend.scene().removeItem(self.legend)
        self.legend = None

    def update_phasor(self, name, am, ph):
        """Change phasor value."""
        self.phasors[name]['end'] = (
            am * math.cos(ph),
            am * math.sin(ph)
        )
        self.__update()

    def __update(self):
        for key in self.phasors:
            phasor = self.phasors[key]
            x = phasor['end'][0]
            y = phasor['end'][1]
            phasor['line'].setData([0, x], [0, y])
            phasor['point'].setData([x], [y])

    def show_legend(self):
        """Show legend."""
        self.legend = self.plotItem.addLegend()
        for key in self.phasors:
            self.plotItem.legend.addItem(
                self.phasors[key]['line'], key)



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Classes for building simple reports with text and pictures."""

import os


class Reporter:
    """Base class for reporter."""
    def __init__(self):
        self._report = ""

    def begin(self):
        """Start the building of report. Some actions may be needed
        fisrt. Make it here."""

    def end(self):
        """Some actions in the end of the process of building."""

    def add_header(self, header_text, level=1):
        """Add header.

        Parameters
        ----------
        header_text : str
            Text of header.
        level : int
            Level of header.
        """
        raise NotImplementedError

    def add_pic(self, pic):
        """Add image to report.

        Parameters
        ----------
        pic : str
            Name of image file.
        """
        raise NotImplementedError

    def add_text(self, text):
        """Add plain text to report.

        Parameters
        ----------
        text : str
            Text
        """
        raise NotImplementedError

    def report(self, file_name, encoding):
        """Save report."""
        raise NotImplementedError


class HtmlReporter(Reporter):
    """HTML reporter."""
    def begin(self):
        """Create the head of html-document and start the body."""
        self._report += "<html>\n<head>"
        self._report += "<Meta charset='UTF-8'/>"
        self._report += "</head>\n<body>\n"

    def end(self):
        """Put close tags to the end of HTML-document."""
        self._report += "</body>\n</html>\n"

    def add_header(self, header_text, level=1):
        """Add header.

        Parameters
        ----------
        header_text : str
            Text of header.
        level : int
            Level of header.
        """
        self._report += "<h{0}>{1}</h{0}>\n".format(level, header_text)

    def add_pic(self, pic):
        """Add image to report.

        Parameters
        ----------
        pic : str
            Name of image file.
        """
        self._report += "<img src='{}' width='800'>\n".format(pic)

    def add_text(self, text):
        """Add plain text to report.

        Parameters
        ----------
        text : str
            Text
        """
        self._report += "<pre>{}</pre>\n".format(text)

    def report(self, file_name, encoding='utf-8'):
        """Save report."""
        with open(file_name, "w", encoding=encoding) as buf:
            buf.write(self._report)


class LatexReporter(Reporter):
    """LaTeX reporter."""
    def add_header(self, header_text, level=1):
        """Add header.

        Parameters
        ----------
        header_text : str
            Text of header.
        level : int
            Level of header.
        """
        if level == 1:
            self._report += "\n\\HeaderCommandOne{"+header_text+"}\n"
        elif level == 2:
            self._report += "\n\\HeaderCommandTwo{"+header_text+"}\n"
        elif level == 3:
            self._report += "\n\\HeaderCommandThree{"+header_text+"}\n"
        else:
            self._report += "\n{"+header_text+"}\n"

    def add_pic(self, pic):
        """Add image to report.

        Parameters
        ----------
        pic : str
            Name of image file.
        """
        self._report += "\n\\InsertPicCommand{"
        self._report += os.path.abspath(os.path.expanduser(pic))
        self._report += "}\n"

    def add_text(self, text):
        """Add plain text to report.

        Parameters
        ----------
        text : str
            Text
        """
        self._report += "\n\\TraceTextCommand{"+text+"}\n"

    def report(self, file_name, encoding='utf-8'):
        """Save report."""
        with open(file_name, "w", encoding=encoding) as buf:
            buf.write(self._report)



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Helpers for CLI."""

import time
import sys


class ProgressMessages:
    """This class provides the console progress messages in format:

    Some operation ... ready [n sec].
    """
    def __init__(self, digits=1):
        self.start_time = None
        self.digits = digits

    def begin(self, message):
        """Show start part of message and three points."""
        self.start_time = time.time()
        sys.stdout.write("{} ... ".format(message))
        sys.stdout.flush()

    def end(self):
        """Show end part of message."""
        end_time = round(time.time()-self.start_time, self.digits)
        sys.stdout.write("ready [{} sec]\n".format(end_time))



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Class for work with JSON config files. """

from collections import OrderedDict
import os
import shutil
import json
import pkg_resources
import logging

LOG = logging.getLogger(__name__)


class Config:
    """ Work with config files: create desired file if it not exists,
    read and save settings. """
    def __init__(self, desired_location, resource_file, package):
        """
        Initiailzation.

        Parameters
        ----------
        desired_location : str
            Name of default file. Full path, may incude '~'. For
            example: '~/config.json'
        resource_file : str
            Package resource file name.
        package : str
            Name of the package.

        """
        self.set_desired_location(desired_location)
        self.resource_file = resource_file
        self.fname = None
        self.package = package
        self.settings = OrderedDict([])
        self._version = None

    def set_desired_location(self, fname):
        """ Set the desired config file location. """
        abspath = os.path.expanduser(fname)
        self.desired_location = abspath

    def use_another_file(self, fname):
        """ Use not default file. """
        self.save_settings()
        self.open(fname)
        try:
            self.read_settings()
            return True
        except Exception as expt:
            msg = "appteka.config: {} raised".format(type(expt))
            LOG.info(msg)
            return False

    def open(self, fname):
        """ Open config file. """
        self.fname = fname

    def open_or_create(self):
        """ Open or create default config file. """
        if not os.path.exists(self.desired_location):
            self._copy_resource_file()
        self.open(self.desired_location)
        return True

    def read_settings(self):
        """ Read settings from file. Settings then can be achieved
        from 'settings' (OrderedDict) field. """
        with open(self.fname) as f:
            json_str = f.read()
        settings = json.loads(json_str, object_pairs_hook=OrderedDict)

        if not self.is_version_valid(settings):
            self.upgrade()
            with open(self.fname) as f:
                json_str = f.read()
            settings = json.loads(json_str, object_pairs_hook=OrderedDict)

        self.settings = settings

        return True

    def save_settings(self):
        """ Save settings to config file. For example, can be used on
        exit program event. """
        json_str = json.dumps(self.settings, indent=4)
        with open(self.fname, "w") as f:
            f.write(json_str)
        return True

    def is_version_valid(self, settings):
        """ Check if version of configuration is more or equal to
        minimal. """
        if self.version is None:
            return True
        if 'version' not in settings.keys() and self.version is None:
            return True
        if 'version' not in settings.keys() and self.version is not None:
            return False
        if 'version' in settings.keys() and self.version is not None:
            if settings['version'] != self.version:
                return False
            else:
                return True
        return True

    def upgrade(self):
        """ Replace config file which have unsupported version newer
        file. """
        self._backup()
        self._copy_resource_file()
        self.open(self.desired_location)

    def _copy_resource_file(self):
        filename = pkg_resources.resource_filename(
            self.package,
            self.resource_file
        )
        shutil.copy(filename, self.desired_location)

    def _backup(self):
        shutil.copy(self.fname, self.desired_location + ".bak")

    def get_version(self):
        """ Return version. """
        return self._version

    def set_version(self, version):
        """ Set version. """
        self._version = version

    version = property(get_version, set_version, doc="The supported version.")



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Implementation of the tool for visual testing of widgets."""

import sys
from warnings import warn

from PyQt5 import QtWidgets

from appteka.pyqt import gui


class TestDialog(QtWidgets.QDialog):
    """Dialog for running unit tests of any widget when human tester
    answers to questions (ok or error)."""
    def __init__(self, parent=None):
        super().__init__(parent)
        self.__make_gui()
        self.widget = None
        self.tests = []
        self.test_num = 0

    def __make_gui(self):
        main_layout = QtWidgets.QVBoxLayout(self)

        # description of test (test name)
        self.descr_layout = gui.add_sublayout(main_layout, "h")
        self.label_descr = gui.add_label("", self.descr_layout)
        self.label_descr.setWordWrap(True)

        # text
        self.assert_layout = gui.add_sublayout(main_layout, "h")
        self.label_assert = gui.add_label("", self.assert_layout)
        self.label_assert.setWordWrap(True)

        # place for widget to be tested
        self.widget_layout = gui.add_sublayout(main_layout, "h")

        # buttons
        self.button_layout = gui.add_sublayout(main_layout, "h")
        self.button_err = gui.add_button(
            "ERROR", self.__on_button_err, self.button_layout)
        self.button_ok = gui.add_button(
            "OK", self.__on_button_ok, self.button_layout)

    def __on_button_ok(self):
        self.tests[self.test_num - 1]['result'] = "OK"
        self.__run_next_test()

    def __on_button_err(self):
        self.tests[self.test_num - 1]['result'] = "ERROR"
        self.__run_next_test()

    def set_widget(self, widget):
        """Sets widget to be tested."""
        if self.widget is not None:
            self.widget_layout.removeWidget(self.widget)
            self.widget.deleteLater()

        self.widget = gui.add_widget(widget, self.widget_layout)

    def set_text(self, value):
        """Deprecated."""
        warn('TestDialog.set_text() deprecated. Use add_assertion.')
        self.label_assert.setText(value)

    def add_assertion(self, line):
        """Add assertion."""
        lines = self.label_assert.text()
        if lines:
            lines += "\n"
        lines += "- {}".format(line)
        self.label_assert.setText(lines)

    def __run_next_test(self):
        if self.test_num == len(self.tests):
            self.__report()
            self.close()
            return

        self._clear_asserts()
        self._set_test_name(self.tests[self.test_num]['name'])

        self.tests[self.test_num]['func']()
        self.test_num += 1

    def run(self):
        """Run all tests."""
        self.__add_tests()

        self.show()

        if not self.tests:
            return

        self.test_num = 0
        self.__run_next_test()

    def _clear_asserts(self):
        self.label_assert.setText("")

    def _set_test_name(self, text):
        self.label_descr.setText("<b>{}</b>".format(text))

    def __report(self):
        print('-----------------------------')
        print(self.__class__.__name__)
        print()
        verdict = 'PASSED'
        for test in self.tests:
            print("{}... {}".format(test['name'], test['result']))
            if test['result'] == 'ERROR':
                verdict = 'NOT PASSED'

        print('-----------------------------')
        print(verdict)
        print()

    def __add_test(self, name):
        test = {
            'func': getattr(self, name),
            'name': name,
            'result': 'ERROR',
        }
        self.tests.append(test)

    def __add_tests(self):
        for fname in dir(self):
            if fname[:5] == "test_":
                if not callable(getattr(self, fname)):
                    continue
                self.__add_test(fname)

    def disable_buttons(self):
        """Disable buttons. For example, for waiting some process
        during the test."""
        self.button_ok.setEnabled(False)
        self.button_err.setEnabled(False)

    def enable_buttons(self):
        """Enable buttons."""
        self.button_ok.setEnabled(True)
        self.button_err.setEnabled(True)


def run(class_name, *args, **kwargs):
    """Run tests.

    Examples
    --------
    >>> class TestSomeWidget(testing.TestDialog):
    >>>     def __init__(self):
    >>>         ...
    >>>     ...
    >>>
    >>> testing.run(TestSomeWidget)

    """
    app = QtWidgets.QApplication(sys.argv)
    class_name(*args, **kwargs).run()
    return app.exec()



import sys
import os

sys.path.insert(0, os.path.abspath("."))
from appteka.pyqt import testing
from appteka.pyqt.codetextedit import CodeTextEdit


class TestCodeTextEdit(testing.TestDialog):
    """Tests for CodeTextEdit"""
    def __init__(self):
        super().__init__()
        self.resize(600, 600)

    def test_text(self):
        e = CodeTextEdit()
        self.set_widget(e)

        code = ""
        code += '{\n'
        code += '  "a": 1,\n'
        code += '  "b": 2\n'
        code += '}'
        e.set_text(code)

        self.add_assertion("Some text is printed")
        self.add_assertion("Lines numbered")
        self.add_assertion("Current line is highlighted")
        self.add_assertion("Font is monospace")


testing.run(TestCodeTextEdit)



# appteka - helpers collection

# Copyright (C) 2018-2020 Aleksandr Popov

# This program is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.

# You should have received a copy of the Lesser GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Select item Qt dialog."""

from PyQt5 import QtWidgets
from appteka.pyqt import gui


class SelectItemDialog(QtWidgets.QDialog):
    """ Dialog for selecting string item from list. """
    def __init__(self, title="Select item", question="Select item",
                 ok_caption="Ok", cancel_caption="Cancel",
                 parent=None):
        QtWidgets.QDialog.__init__(self, parent)

        self._items = []
        self._index = None

        self._make_gui(title, question, ok_caption, cancel_caption)

    def _make_gui(self, title, question, ok_caption, cancel_caption):
        self.setWindowTitle(title)
        main_layout = QtWidgets.QVBoxLayout(self)
        gui.add_label(question, main_layout)
        self._quant_list = gui.add_widget(QtWidgets.QListWidget(), main_layout)
        self._quant_list.setWordWrap(True)
        self._quant_list.setSpacing(2)

        buttons_layout = gui.add_sublayout(main_layout, 'h')
        gui.add_button(ok_caption, self._ok_clicked,
                       buttons_layout)
        gui.add_button(cancel_caption, self._cancel_clicked,
                       buttons_layout)

    def exec(self):
        """ Execute dialog. """
        for item in self._items:
            self._quant_list.addItem(item)
        return super().exec()

    def _ok_clicked(self):
        """ OK button clicked handler. """
        self._index = self._quant_list.currentRow()
        self.accept()

    def _cancel_clicked(self):
        """ Cancel button clicked handler. """
        self.reject()

    def set_items(self, str_list):
        """ Set the names of items. """
        self._items = str_list

    def get_item_name(self):
        """ Return name selected item. """
        ind = self.get_item_index()
        name = self._items[ind]
        return name

    def get_item_index(self):
        """ Return index of selected item. """
        if self._index is None:
            raise RuntimeError("Index is None.")
        ind = self._index
        return ind
